''***************************************
''*  Graphics Demo                      *
''*  Author: Chip Gracey                *
''*  Copyright (c) 2005 Parallax, Inc.  *               
''*  See end of file for terms of use.  *               
''***************************************


CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  _stack = ($3000 + $3000 + 100) >> 2   'accomodate display memory and stack


'  x_tiles = 16
'  y_tiles = 12

  'paramcount = 14       
  bitmap_base = $2000
  display_base = $5000

  lines = 5
  thickness = 2

  'Transcevier Pins

  xcvrdat       = 3    'Data
  TRsel         = 2    ' Trans Recieve Select (1 = Trans, 0=Rec)
  xcvrPDN       = 1    ' Power 5 down 0 = low current state
  Speed          = 9000 ' Serial baud rate

  ' A to D Pins
  CL            = 19 'Clock         {P19}
  DI            = 18 'Data In       {P18}
  DO            = 21 'Data Out      {P21}
  CS            = 20 'Chip Select  {P20}
  ' 74HC597 pins
  sload   = 15
  rck     = 14
  sck     = 23
  sclr    = 16
  Qh      = 12



  x_tiles = 16
  y_tiles = 12

  x_screen = x_tiles << 4
  y_screen = y_tiles << 4

  width = 0             '0 = minimum
  x_scale = 1           '1 = minimum
  y_scale = 1           '1 = minimum
  x_spacing = 6         '6 = normal
  y_spacing = 13        '13 = normal

  x_chr = x_scale * x_spacing
  y_chr = y_scale * y_spacing

  y_offset = y_spacing / 6 + y_chr - 1

  x_limit = x_screen / (x_scale * x_spacing)
  y_limit = y_screen / (y_scale * y_spacing)
  y_max = y_limit - 1

  y_screen_bytes = y_screen << 2
  y_scroll = y_chr << 2
  y_scroll_longs = y_chr * y_max
  y_clear = y_scroll_longs << 2
  y_clear_longs = y_screen - y_scroll_longs

  paramcount = 14  
  GraphicsWait = 300_000

VAR
  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

  word  screen[x_tiles * y_tiles]
  long  colors[16]
  long  x,y

  byte ChecksumIN
  byte Checksum
  
  byte Dat0,DatNew0           ' Inbound data byte 0 
  byte Dat1,DatNew1           ' Inbound data byte 1
  byte Dat2,DatNew2           ' Inbound data byte 2
  byte Dat3,DatNew3           ' Inbound data byte 3

  long Tx[4] , ChecksumT, ChecksumR
  long CM0,CM1
  word CamAngle,LoopCount
  byte rxbyte,rxTest,OK2Transmit
  long Temp , CamStat, i
  long TVCog,SerCog,NumCog,GrCog,LtCog
  LONG TransWait
  word angle
  long LatchData
  
  long ADStack[16]   
  word ADCog,Span[8],ADDefl[8] ,AnalogData[8]
  
  
OBJ  
    tv    : "tv"
    gr    : "graphics" 
    Num   : "NumbersNEW"         
'    Ser   : "PWMSerial"  
    AD    : "ADC088S052DACog"
    Lt    : "74HC597"   

PUB start |  j, k, kk, dx, dy, pp, pq, rr, numx, numchr
  ' ========================== Initialize Number handling routines ====================
  Num.Init         'Initialize Numbers - No cogs started but left for consistancy
  ' ========================== Start TV & Graphics routines ====================    
  'start tv
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)

  'init colors
  repeat i from 0 to 15
    colors[i] := $00001010 * (i+8) & $F + $2B060C02

  'init tile screen
  repeat dx from 0 to tv_hc - 1
    repeat dy from 0 to tv_vc - 1
      screen[dy * tv_hc + dx] := display_base >> 6 + dy + dx * tv_vc + ((dy & $3F) << 10)
      
  'start and setup graphics

  gr.start
  gr.setup(x_tiles, y_tiles, 0, y_screen, bitmap_base)
  gr.textmode(x_scale, y_scale, x_spacing, 0)
  gr.width(width)

    ' ========================== Start data Converter routines ====================
    ADCog := AD.Start(CL,DI,DO,CS)    ' This includes AD
  ' =========================== Set up latch cog ==========================
    LtCog := Lt.Start(sload,rck,sck,sclr,Qh)' This includes 32bit switch latch     
  ' ========================== Set up ports for RF communication and prep Tx & Rx in Start  '
'    SerCog := Ser.init(xcvrdat, xcvrdat, Speed, TRsel, xcvrPDN)      '(rxPin, txPin, baud)

   ' ========================== Indicator LED setup =========================
   Dira[26] := 1               ' Green LED output
   Dira[27] := 1               ' Red LED output
   Outa[26] := 0               ' Green Off
   outa[27] := 0               ' Red off
   ' ========================== End Indicator LED setup =====================

  Main

Pub Main

  repeat
    LatchData :=Lt.GetLatch
    LoadADData                  ' Get data from A to D's       
                  

    repeat i from 0 to 7
      ADDefl[i] := AnalogData[i] 

    GreenON
    TransmitData
    RecieveData
    GreenOFF
    MainScreen    ' Draw data display

    
PUB TransmitData   '' Transmits one data package form 255,0,d,d,d,d,checksum 
    'ser.tx(Temp, 2)

PUB RecieveData    '' Recieves one data package form 255,0,d,d,d,d,checksum
'    dat0 := ser.rx(1)
'    dat1 := ser.rx(2)
'    dat2 := ser.rx(3)
'    dat3 := ser.rx(4)
          
PUB LoadADData                       ' Loads 8 data samples from AD converter 
  repeat i from 0 to 7               ' Returns 0 to 100
    AnalogData[i] := AD.GetAD(i)   

Pub TextLine(Xcol,Yline,TxtPtr)
    gr.text(x_spacing * Xcol,-y_Spacing + (-y_spacing * YLine),TxtPtr )
            
pub DisplayData (Xcol,Yline,Data)  ' ------------------- Works ------------------------  
    gr.text(x_spacing * Xcol,-y_Spacing + (-y_spacing * YLine),num.ToStr(Data, Num#DEC))

pub DisplayDataFine (Xp,Yp,Data)  ' ------------------- Works ------------------------  
    gr.text(Xp,Yp,num.ToStr(Data, Num#DEC))
    
PUB DisplayBin (Xcol,Yline,Data)   
    gr.text(x_spacing * Xcol,-y_Spacing + (-y_spacing * YLine),num.ToStr(Data, Num#DBIN40))

pub DisplayString(Xpos,Ypos,Color,StrPtr)   ' Displays Charactor ASCII val Char
    gr.color(Color)                         ' Color 1=Red, 2=White, 3=blue, 
    gr.text(Xpos, Ypos , @StrPtr)

pub GreenON
    Outa[26] := 1               ' Green LED On
pub GreenOFF
    Outa[26] := 0               ' Green LED Off 
pub RedON
   ' outa[27] := 1               ' Red LED On
pub REDOFF
   ' outa[27] := 0               ' Red LED off
pub MainScreen

    'clear bitmap
    gr.clear
    gr.color(3)  
    TextLine(0,0,(string("===Cannibal Robotics Human Interface v 1.3===")+0))
    gr.color(1)
    
    gr.color(2)

    DisplayBIN(1,13,LatchData)
    gr.finish
    ' ADDefl[i]                                                
    ArcGuage( 30,-50,ADDefl[7],(string("Thro")+0),0)
    ArcGuage( 93,-50,ADDefl[0],(string("Thm")+0),0)
    ArcGuage(156,-50,ADDefl[5],(string("Fgr")+0),0)
    ArcGuage(219,-50,ADDefl[6],(string("Rud")+0),0)

    ArcGuage( 30,-120,ADDefl[3],(string("X")+0),0)
    ArcGuage( 93,-120,ADDefl[4],(string("Y")+0),0)
    'ArcGuage(156,-120,AnalogData[6],(string("AD6")+0),0)
    'ArcGuage(219,-120,AnalogData[7],(string("AD7")+0),0)


    gr.copy(display_base)

PUB ArcGuage(x_pos,y_pos,Deflection,lblptr,datapoint)                    ' Draws Arc shapped guage
  Angle := Deflection * 62
  gr.width(1)
  gr.color(3)
  gr.plot(x_pos,y_pos)
  gr.arc(x_pos, y_pos, 30, 30, 0 ,100 , 62, 1)  
  gr.plot(x_pos,y_pos) 
  gr.color(6)
  gr.arc(x_pos, y_pos, 30, 30, Angle ,100, 1, 1)
  gr.width(0)
  DisplayDataFine (x_pos+10,y_pos-18,deflection)
  gr.color(1)  
  gr.text(x_pos+10,y_pos-30,lblptr)

     'copy bitmap to display
  gr.finish

DAT
tvparams                long    0               'status
                        long    1               'enable
                        long    %000_0101       'pins
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    0               'broadcast
                        long    0               'auralcog

pchip                   byte    "Propellor",0           'text